vec3 get_lava_fog(float dist, vec3 color) {
    const vec3 LAVA_FOG_COLOR = vec3(0.3, 0.09, 0.01);
    const vec3 PSNOW_FOG_COLOR = vec3(0.2, 0.4, 0.5);
    
    switch(isEyeInWater) {
        case 1:
            vec3 UnderwaterCol = vec3(WATER_RED, WATER_GREEN, WATER_BLUE);
            dist = clamp(dist/128, 0, 1);
            dist = 1.0 - exp(-3.0 * dist);
            return mix(color, UnderwaterCol, dist);
        case 2:
            dist = clamp(dist/2, 0, 1);
            return mix(color, LAVA_FOG_COLOR, dist);
        case 3:
            dist = clamp(dist/2, 0, 1);
            return mix(color, PSNOW_FOG_COLOR, dist);
    }
    return color;
}

vec3 get_border_fog(float strength, vec3 color, vec3 SkyColor) {
    strength *= strength;
    #ifndef DIMENSION_NETHER
    strength *= strength;
    strength *= strength;
    #endif
    strength = 1.0 - exp(-3.0 * strength); 
    strength = clamp(strength, 0, 1);
    return mix(color, SkyColor, strength);
}

vec3 get_blindness_fog(float Dist, vec3 Color) {
    if(darknessFactor/4 > blindness) Dist /= 4;
    Dist = clamp(1.0 - exp(-3.0 * Dist/10), 0, 1)*max(darknessFactor, blindness);
    return Color * (1-Dist);
}

vec3 get_atm_fog(float Dist, vec3 Color, vec3 WorldPos, vec3 FogColor) {
    Dist = min(Dist/256, 1);
    Dist = 1.0 - exp(-3.0 * Dist);
    float Visibility = sunriseStrength*0.7+nightStrength;
    Visibility = max(Visibility, rainStrength)*is_indoors_smooth();
    Visibility *= ATM_FOG_STRENGTH;

    //FogColor = apply_saturation(FogColor, 0.5);
    float HeightFalloff = WorldPos.y >= 40 ? smoothstep(40, 60, WorldPos.y)-smoothstep(60, 90, WorldPos.y) : 0;
    vec2 Animation = WorldPos.xz+frameTimeCounter*3;
    float Factor = Dist*HeightFalloff*sqrt(fbm_fast(Animation/32, 2))*Visibility;
    vec3 FinalC = mix(Color, FogColor, Factor);
    return FinalC;
}
