
// Returns the bloom tile you are currently on
int is_in_range(vec2 texcoord) { 
    vec2 accepted_range[2] = vec2[2](vec2(0), vec2(0));
    for(int i = 2; i <= 2+BLOOM_SIZE; i++) {
        accepted_range[0] = accepted_range[1];
        accepted_range[1] += 1/pow(2, i);
        if(all(greaterThan(texcoord, accepted_range[0])) && all(lessThan(texcoord, accepted_range[1]))) {
            return i;
        }
    }
    return 0;
}

// https://github.com/Experience-Monks/glsl-fast-gaussian-blur
vec3 blur(sampler2D image, vec2 uv, vec2 direction) {
    vec3 color = vec3(0.0);
    vec2 off1 = vec2(1.3333333333333333) * direction / resolution;
    color += texture2D(image, uv).rgb * 0.29411764705882354;
    if(is_in_range(uv + off1) != 0)
        color += texture2D(image, uv + off1).rgb * 0.35294117647058826;
    if(is_in_range(uv - off1) != 0)
        color += texture2D(image, uv - off1).rgb * 0.35294117647058826;
    return color; 
}


