// In shaderlabs discord: https://discord.com/channels/237199950235041794/525510804494221312/955506913834070016
vec2 toPrevScreenPos(vec2 currScreenPos, float depth){
    vec3 currViewPos = vec3(vec2(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y) * (currScreenPos.xy * 2.0 - 1.0) + gbufferProjectionInverse[3].xy, gbufferProjectionInverse[3].z);
    currViewPos /= (gbufferProjectionInverse[2].w * (depth * 2.0 - 1.0) + gbufferProjectionInverse[3].w);
    vec3 currFeetPlayerPos = mat3(gbufferModelViewInverse) * currViewPos + gbufferModelViewInverse[3].xyz;

    vec3 prevFeetPlayerPos = depth > 0.56 ? currFeetPlayerPos + cameraPosition - previousCameraPosition : currFeetPlayerPos;
    vec3 prevViewPos = mat3(gbufferPreviousModelView) * prevFeetPlayerPos + gbufferPreviousModelView[3].xyz;
    vec2 finalPos = vec2(gbufferPreviousProjection[0].x, gbufferPreviousProjection[1].y) * prevViewPos.xy + gbufferPreviousProjection[3].xy;
    return (finalPos / -prevViewPos.z) * 0.5 + 0.5;
}

vec3 RGB_to_YCoCg(vec3 ip) {
    const mat3 CONVERSION_MAT = mat3(
        1./4, 1./2, -1./4,
        1./2, 0, 1./2,
        1./4, -1./2, -1./4
    );
    return CONVERSION_MAT*ip;
}

vec3 YCoCg_to_RGB(vec3 ip) {
    float tmp = ip.x-ip.z;
    return vec3(
        tmp + ip.y,
        ip.x+ip.z,
        tmp-ip.y
    );
}

// https://discord.com/channels/237199950235041794/525510804494221312/955458285367066654
vec3 clipAABB(vec3 prevColor, vec3 minColor, vec3 maxColor) {
    vec3 pClip = 0.5 * (maxColor + minColor); // Center
    vec3 eClip = 0.5 * (maxColor - minColor); // Size

    vec3 vClip  = prevColor - pClip;
    vec3 aUnit  = abs(vClip / eClip);
    float denom = max(aUnit.x, max(aUnit.y, aUnit.z));

    return denom > 1.0 ? pClip + vClip / denom : prevColor;
}

vec3 neighbourhoodClipping(sampler2D currTex, vec3 prevColor) {
    vec3 minColor = vec3(1e5), maxColor = vec3(-1e5);
    prevColor = RGB_to_YCoCg(prevColor);

    for(int x = -1; x <= 1; x++) {
        for(int y = -1; y <= 1; y++) {
            if(x == y && x == 0) continue;
            vec3 color = RGB_to_YCoCg(texture2D(currTex, (gl_FragCoord.xy + vec2(x, y))/resolution).rgb);
            minColor = min(minColor, color); 
            maxColor = max(maxColor, color); 
        }
    }
    return YCoCg_to_RGB(clipAABB(prevColor, minColor, maxColor));
}

vec3 TAA(inout vec3 Color) {
    float Depth = texture2D(depthtex1, texcoord).x;
    vec2 PrevCoord = toPrevScreenPos(texcoord, Depth);
    vec3 PrevColor = texture2D(colortex3, PrevCoord).rgb;
    if(PrevColor == vec3(0)) return Color;
    vec3 ClampedColor = neighbourhoodClipping(colortex0, PrevColor);

    vec2 velocity = (texcoord - PrevCoord.xy) * resolution;
	float blendFactor = float(
		PrevCoord.x > 0.0 && PrevCoord.x < 1.0 &&
		PrevCoord.y > 0.0 && PrevCoord.y < 1.0
	);
	blendFactor *= exp(-length(velocity)) * 0.3 + 0.6;

    Color = mix(Color, ClampedColor, blendFactor);
    return Color;
}
