// Very simple and inaccurate, but it's better than nothing.
float water_fog() {
    float TerrainDepth = texture2D(depthtex1, gl_FragCoord.xy/resolution).x;
    TerrainDepth = linearize_depth(TerrainDepth);
    float ScreenDepth = linearize_depth(gl_FragCoord.z);
    return smoothstep(0, 64, TerrainDepth-ScreenDepth)*WATER_FOG_STRENGTH;
}

float schlick(vec3 V, vec3 N) {
    const float R = 0.1;
    float Theta = 1-dot(-V, N);
    Theta = max(Theta, 0); // https://www.youtube.com/watch?v=BcxKINWMD8M
    return R + (1-R)*(pow(Theta, 5.0));
}

vec3 get_water_normal(vec2 Coords) {
	float Offset = 0.3;
	float D0 = noise_water(Coords);
	float D1 = noise_water(vec2(Coords.x+Offset, Coords.y));
	float D2 = noise_water(vec2(Coords.x, Coords.y+Offset));
    float A = (D0-D1);
    float B = (D0-D2);
	return normalize(vec3(A, B, WATER_NORMAL_STRENGTH));
}

vec4 get_water_color(float SkyBrightness) {
    SkyBrightness = max(SkyBrightness, MIN_LIGHT_AMOUNT);
    return vec4(vec3(WATER_RED, WATER_GREEN, WATER_BLUE)*SkyBrightness, WATER_ALPHA);
}

vec3 sky_reflection(vec3 ReflectedVec, float Dist) {
    #ifdef DIMENSION_OVERWORLD
    vec3 Reflection = get_sky(ReflectedVec, Dist);
    Reflection.rgb += round_sun(Dist)*4;
    return Reflection;
    #else
    return fogColor.rgb;
    #endif
}

vec3 ssr(vec3 RVec, float Dist, vec3 ViewPos) {
    vec3 ScreenPos = gl_FragCoord.xyz/vec3(resolution, 1);
    // Convert RVec to screenspace
    vec3 Offset = normalize(view_screen(ViewPos+RVec)-ScreenPos)/SSR_STEPS;
    #if TAA_MODE == 0
	float Noise = random(ScreenPos.xy);
	#else
	float Noise = random(ScreenPos.xy+framemod8);
	#endif
    vec3 ExpectedPos = ScreenPos+Offset*Noise;
    for(int i = 1; i <= SSR_STEPS; i++) {
        
        // Stretch the reflections a bit horizontally
        if(ExpectedPos.x < -0.1 || ExpectedPos.y < 0 || ExpectedPos.x > 1.1 || ExpectedPos.y > 1) {
            return sky_reflection(RVec, Dist);
        }
        
        float RealDepth = texture2D(depthtex1, ExpectedPos.xy).x;
        if(RealDepth >= 1 || RealDepth < 0.56) {
            return sky_reflection(RVec, Dist);
        }
        
        if(ExpectedPos.z > RealDepth) {
            // Binary refinement
            for(int i = 1; i <= 3; i++) {
                Offset /= 2;
                vec3 EPos1 = ExpectedPos-Offset;
                float RDepth1 = texture2D(depthtex1, EPos1.xy).x;
                if(EPos1.z > RDepth1) {
                    ExpectedPos = EPos1;
                }
            }
            return texture2D(colortex4, ExpectedPos.xy).rgb;
        }
        ExpectedPos += Offset;
    }
    return sky_reflection(RVec, Dist);
}

vec4 get_fancy_water(vec3 ViewPos, float SkyBrightness) {
    vec3 ViewPosN = normalize(ViewPos);
    vec4 BaseColor = get_water_color(SkyBrightness);
    BaseColor.a = min(BaseColor.a+water_fog(), 1);

    vec3 WorldPos = to_player_pos(ViewPos)+cameraPosition;

    vec3 NormalMap = get_water_normal(vec2(WorldPos.x, WorldPos.z));
    vec3 WaterNormal = TBN*NormalMap;

    vec3 ReflectedVec = reflect(ViewPosN, WaterNormal);
    float Dist = distance(ReflectedVec, normalize(sunPosition));

    #ifdef SSR
    vec3 Reflection = ssr(ReflectedVec, Dist, ViewPos);
    #else
    vec3 Reflection = sky_reflection(ReflectedVec, Dist);
    #endif

    BaseColor.rgb = mix(BaseColor.rgb, Reflection, schlick(ViewPosN, WaterNormal)*max(SkyBrightness, 0));
    
    return BaseColor;
}
