float random(vec2 coords) {
   return fract(sin(dot(coords.xy, vec2(12.9898,78.233))) * 43758.5453);
}

vec3 dither(vec3 Color, vec2 Pos) {
    Color += (random(Pos.xy)-0.5)/256;
    return Color;
}

vec3 project_and_divide(mat4 Projection_mat, vec3 x) {
    vec4 HomogeneousPos = Projection_mat * vec4(x, 1);
    return HomogeneousPos.xyz / HomogeneousPos.w;
}

vec3 to_view_pos(vec3 p) {
	p = p * 2 - 1;
    return project_and_divide(gbufferProjectionInverse, p);

}

vec3 view_screen(vec3 x) {
    x = project_and_divide(gbufferProjection, x);
    x = x*0.5 + 0.5;
    return x;
}

vec3 to_player_pos(vec3 p) {
    return mat3(gbufferModelViewInverse) * p;
}

float is_indoors_smooth() {
    return smoothstep(5, 50, float(eyeBrightnessSmooth.y));
}

// Code by Jessie: https://github.com/Jessie-LC/open-source-utility-code/blob/main/advanced/packing.glsl

vec2 encodeUnitVector(vec3 vector) {
	// Scale down to octahedron, project onto XY plane
	vector.xy /= abs(vector.x) + abs(vector.y) + abs(vector.z);
	// Reflect -Z hemisphere folds over the diagonals
	return vector.z <= 0.0 ? (1.0 - abs(vector.yx)) * vec2(vector.x >= 0.0 ? 1.0 : -1.0, vector.y >= 0.0 ? 1.0 : -1.0) : vector.xy;
}

vec3 decodeUnitVector(vec2 encoded) {
	// Exctract Z component
	vec3 vector = vec3(encoded, 1.0 - abs(encoded.x) - abs(encoded.y));
	// Reflect -Z hemisphere folds over the diagonals
	float t = max(-vector.z, 0.0);
	vector.xy += vec2(vector.x >= 0.0 ? -t : t, vector.y >= 0.0 ? -t : t);
	// Normalize and return
	return normalize(vector);
}

float linearize_depth(float D) {
    return near / (1-D);
}

// Creates a TBN matrix from a normal and a tangent
mat3 tbnNormalTangent(vec3 normal, vec3 tangent) {
	// For DirectX normal mapping you want to switch the order of these
	vec3 bitangent = cross(normal, tangent);
	return mat3(tangent, bitangent, normal);
}

// Creates a TBN matrix from just a normal
// The tangent version is needed for normal mapping because
//   of face rotation
mat3 tbnNormal(vec3 normal) {
	// This could be
	// normalize(vec3(normal.y - normal.z, -normal.x, normal.x))
	vec3 tangent = normalize(cross(normal, vec3(0, 1, 1)));
	return tbnNormalTangent(normal, tangent);
}

uniform float framemod8;

vec2 jitterOffsets[8] = vec2[8](
							vec2( 0.125,-0.375),
							vec2(-0.125, 0.375),
							vec2( 0.625, 0.125),
							vec2( 0.375,-0.625),
							vec2(-0.625, 0.625),
							vec2(-0.875,-0.125),
							vec2( 0.375,-0.875),
							vec2( 0.875, 0.875)
						);
