#include "/lib/all_the_libs.glsl"
varying vec2 texcoord;

#include "/global/post/bloom.glsl"
#include "/global/post/taa.glsl"

#extension GL_EXT_gpu_shader4 : enable

// Color grading, Bloom final, TAA final

vec3 film_grain(vec3 Color, vec2 Pos) {
    Pos.x += fract(float(frameCounter)/4.14159)*234;
    Pos.y -= fract(float(frameCounter)/5.49382)*567;
    Color += (texture2D(noisetex, vec2(ivec2(Pos.xy) & 255)/255).rgb - 0.5)*FILM_GRAIN_STRENGTH;
    return Color;
}

vec3 apply_vignette(vec3 Color, vec2 Pos) {
	Pos = Pos - 0.5;
	float Strength = length(Pos*VIGNETTE_OPACITY);
	Strength = pow(Strength, 4-VIGNETTE_FALLOFF);
	Color *= 1-min(Strength, 1);
	return Color;
}

/* DRAWBUFFERS:03 */

void main() {
	vec4 Color = texture2D(colortex0, texcoord);
	#if TAA_MODE != 0
	gl_FragData[1].xyz = TAA(Color.rgb);
	#endif
	#ifdef BLOOM 
		float Offset = 0;
		vec3 FinalBloom = vec3(0);
		for(int i = 2; i <= 2+BLOOM_SIZE; i++) {
			FinalBloom += texture2D(colortex1, texcoord/pow(2, i)+Offset).xyz;
			Offset += 1/pow(2, i);
		}
		FinalBloom /= BLOOM_SIZE;
		Color.rgb = mix(Color.rgb, FinalBloom, clamp(get_luminance(FinalBloom)-BLOOM_THRESHOLD, 0, 0.5)*BLOOM_STRENGTH);
	#endif

	Color.rgb *= EXPOSURE;
	Color.rgb = ACESFilm(Color.rgb);
	Color.rgb = pow(Color.rgb, vec3(1/2.2)); // Gamma correction
	Color.xyz = dither(Color.xyz, gl_FragCoord.xy);

	Color.rgb = apply_vibrance(Color.rgb, VIBRANCE);
	Color.rgb = apply_saturation(Color.rgb, SATURATION);
	Color.rgb = apply_contrast(Color.rgb, CONTRAST);

	vec3 MinBright = vec3(TONEMAP_MIN_R, TONEMAP_MIN_G, TONEMAP_MIN_B);
	Color.rgb = max(Color.rgb, MinBright);

	#ifdef FILM_GRAIN
	Color.rgb = film_grain(Color.rgb, gl_FragCoord.xy);
	#endif

	Color.rgb = apply_vignette(Color.rgb, texcoord);

	gl_FragData[0] = Color;
	#ifdef DEBUG_BUFFER
	gl_FragData[0] = texture2D(colortex1, texcoord);
	#endif
}
