#include "/lib/all_the_libs.glsl"

#include "/global/sky.glsl"
#include "/global/fog.glsl"
#include "/global/post/ssao.glsl"

// Sky, Clouds, etc

varying vec2 texcoord;

/* DRAWBUFFERS:014 */

void main() {
	vec4 Color = texture2D(colortex0, texcoord);
	float Depth = texture2D(depthtex0, texcoord).x;

	vec3 ViewPos = to_view_pos(vec3(texcoord, Depth));
	vec3 PlayerPos = mat3(gbufferModelViewInverse) * ViewPos;

	#ifdef DIMENSION_OVERWORLD
	vec3 ViewPosN = normalize(ViewPos);
	float Dist = distance(ViewPosN, normalize(sunPosition));
	vec3 SkyColor = get_sky(ViewPosN, Dist);
	#elif defined DIMENSION_END
	vec3 SkyColor = get_end_sky(normalize(ViewPos), normalize(PlayerPos));
	#else
	float LightenFactor = 0.8 - get_luminance(fogColor.xyz); // Makes all biomes roughly equally bright
	LightenFactor *= LightenFactor;
	vec3 SkyColor = apply_saturation(fogColor, 1.2)*LightenFactor;
	#endif
	gl_FragData[1].rgb = SkyColor;

	if(Depth >= 1) {
		#ifndef DIMENSION_OVERWORLD
		Color = vec4(0, 0, 0, 1);
		#elif defined ROUND_SUN
		SkyColor += round_sun(Dist);
		#endif
		Color.xyz += SkyColor;

		if(PlayerPos.y > 0) {
			#if defined DIMENSION_OVERWORLD || defined DIMENSION_END
			Color.xyz += get_stars(PlayerPos);
			#endif

			#if defined FANCY_CLOUDS && defined DIMENSION_OVERWORLD
				vec3 CloudPos = normalize(PlayerPos);
				CloudPos.xz = PlayerPos.xz / (PlayerPos.y + length(PlayerPos.xz)/4);
				CloudPos.xz += cameraPosition.xz/512;
				Color.xyz = get_clouds(CloudPos, Color.xyz);
			#endif
		}

	}
	
	else if(Depth >= 0.56) { // Skip hand & sky
		#ifdef SSAO
		vec3 Normal = decodeUnitVector(texture2D(colortex2, texcoord).rg * 2 - 1);
		Color.rgb = ssao(Color.rgb, Normal, ViewPos);
		#endif
		#ifdef SSR
		gl_FragData[2] = Color;
		#endif
	}
	
	gl_FragData[0] = Color;
}

